-- VT_Mon.vhd
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity VT_Mon is
port (CLK40M, SYS_RESET, INIT_POWER_UP, ATRETURN01: in std_logic;
      AV5V, Temp: in std_logic;
      VAREF: inout std_logic;
      Supply_good, Over_temp: out std_logic);
end VT_Mon;

architecture RTL of VT_Mon is

-- internal signals
signal INIT_DONE_net: std_logic;
signal INIT_DONEn_net: std_logic;
signal SYS_CLK_net: std_logic;
signal SYS_CLK_int: std_logic;
signal SYS_RESET_net: std_logic;
signal VAREF_net: std_logic;
signal AV5V_net: std_logic;
signal Temp_net: std_logic;
signal Supply_good_net: std_logic;
signal Supply_good_en_net: std_logic;
signal Over_temp_en_net: std_logic;
signal Over_temp_net: std_logic;
signal ATRETURN01_net: std_logic;
signal AV5V_over5p25_net: std_logic;
signal AV5V_under4p75_net: std_logic;
signal Temp_over70C_net: std_logic;
signal INIT_ADDR_net: std_logic_vector(8 downto 0);
signal INIT_DATA_net: std_logic_vector(8 downto 0);
signal INIT_ACM_WEN_net: std_logic;
signal INIT_ASSC_WEN_net: std_logic;
signal INIT_EV_WEN_net: std_logic;
signal INIT_TR_WEN_net: std_logic;
signal INIT_POWER_UP_net: std_logic;
signal CLK10M: std_logic;               -- 10 MHz clock for initialization and ACMCLK
signal PLL_CLK40M: std_logic;           -- 40 MHz clock from PLL

-- component declarations
component Analog is
port( SYS_CLK        : in    std_logic;
          SYS_RESET      : in    std_logic;
          VAREF          : inout std_logic := 'Z';
          DATAVALID      : out   std_logic;
          AV5V           : in    std_logic;
          Temp           : in    std_logic;
          Supply_good    : out   std_logic;
          Supply_good_en : in    std_logic;
          Over_temp      : out   std_logic;
          Over_temp_en   : in    std_logic;
          ATRETURN01     : in    std_logic;
          Temp_over70C   : out   std_logic;
          AV5V_over5p25  : out   std_logic;
          AV5V_under4p75 : out   std_logic;
          ASSC_SEQIN     : in    std_logic_vector(5 downto 0);
          ASSC_XMODE     : in    std_logic;
          ASSC_XTRIG     : in    std_logic;
          ASSC_SEQJUMP   : in    std_logic;
          ASSC_DONE      : out   std_logic;
          ASSC_WAIT      : out   std_logic;
          ASSC_CHSAT     : out   std_logic;
          ASSC_CHLATD    : out   std_logic;
          ASSC_SEQCHANGE : out   std_logic;
          ASSC_SEQOUT    : out   std_logic_vector(5 downto 0);
          ASSC_SAMPFLAG  : out   std_logic;
          INIT_ADDR      : in    std_logic_vector(8 downto 0);
          INIT_DATA      : in    std_logic_vector(8 downto 0);
          INIT_ACM_WEN   : in    std_logic;
          INIT_ASSC_WEN  : in    std_logic;
          INIT_EV_WEN    : in    std_logic;
          INIT_TR_WEN    : in    std_logic;
          INIT_DONE      : in    std_logic;
          ADC_CHNUMBER   : out   std_logic_vector(4 downto 0);
          ACMCLK         : in    std_logic;
          ACMRDATA_I     : out   std_logic_vector(7 downto 0)
          );
end component Analog;

component NVM_sys is
port( INIT_CLK      : in    std_logic;
          SYS_RESET     : in    std_logic;
          INIT_POWER_UP : in    std_logic;
          INIT_DONE     : out   std_logic;
          INIT_DATA     : out   std_logic_vector(8 downto 0);
          INIT_ADDR     : out   std_logic_vector(8 downto 0);
          INIT_ACM_WEN  : out   std_logic;
          INIT_ASSC_WEN : out   std_logic;
          INIT_EV_WEN   : out   std_logic;
          INIT_TR_WEN   : out   std_logic
        );   
end component NVM_sys;

component NGM is 
port(CLK0, CLK1, S : in std_logic;  
     GL : out std_logic) ;
end component NGM;

component PLL_40_10 is 
port(POWERDOWN, CLKA, OADIVRST : in std_logic;
     LOCK, GLA, GLC : out std_logic) ;
end component PLL_40_10;

begin

-- connections to top level ports
SYS_RESET_net <= SYS_RESET;
VAREF_net <= VAREF;
AV5V_net <= AV5V;
Temp_net <= Temp;
ATRETURN01_net <= ATRETURN01;
Supply_good <= Supply_good_net;
Over_temp <= Over_temp_net;
INIT_POWER_UP_net <= INIT_POWER_UP;

-- internal assignments
INIT_DONEn_net <= not Init_Done_net;

-- divide 40 MHz input clock by 4 to generate 10 MHz init clock
PLL_inst: PLL_40_10 port map (POWERDOWN => '1', CLKA => CLK40M, OADIVRST => '1', LOCK => open, 
                              GLA => PLL_CLK40M, GLC => CLK10M);

-- multiplex CLK10M and SYS_CLK for initialization
NGMUX_inst: NGM port map(CLK0 => PLL_CLK40M, CLK1 => CLK10M, S => INIT_DONEn_net, GL => SYS_CLK_int);

-- generate enable signals for gate drivers

Over_temp_en_net <= Temp_over70C_net;   -- use over temp flag to turn on gate driver

Supply_good_en_net <= not AV5V_under4p75_net and not AV5V_over5p25_net;

-- instantiate Analog block
Analog_inst: Analog port map
                   (SYS_CLK => SYS_CLK_int, 
                    SYS_RESET => SYS_RESET_net, 
                    VAREF => VAREF_net,
                    DATAVALID => open, 
                    AV5V => AV5V_net,
                    Temp => Temp_net, 
                    Supply_good => Supply_good_net,
                    Supply_good_en => Supply_good_en_net,
                    Over_temp => Over_temp_net,
                    Over_temp_en => Over_temp_en_net, 
                    ATRETURN01 => ATRETURN01_net,
                    Temp_over70C => Temp_over70C_net, 
                    AV5V_over5p25 => AV5V_over5p25_net,
                    AV5V_under4p75 => AV5V_under4p75_net,
                    ASSC_SEQIN => (others =>'0'),
                    ASSC_XMODE => '0',
                    ASSC_XTRIG => '0',
                    ASSC_SEQJUMP => '0',
                    ASSC_DONE => open,
                    ASSC_WAIT => open,
                    ASSC_CHSAT => open,
                    ASSC_CHLATD => open,
                    ASSC_SEQCHANGE => open,
                    ASSC_SEQOUT => open,
                    ASSC_SAMPFLAG => open,
                    INIT_ADDR => INIT_ADDR_net, 
                    INIT_DATA => INIT_DATA_net,
                    INIT_ACM_WEN => INIT_ACM_WEN_net,
                    INIT_ASSC_WEN => INIT_ASSC_WEN_net,
                    INIT_EV_WEN => INIT_EV_WEN_net, 
                    INIT_TR_WEN => INIT_TR_WEN_net,
                    INIT_DONE => INIT_DONE_net, 
                    ADC_CHNUMBER => open,
                    ACMCLK => CLK10M,
                    ACMRDATA_I => open
                    );

-- instantiate Flash Memory block
NVM_inst: NVM_sys port map
                (INIT_CLK => SYS_CLK_int,
                 SYS_RESET => SYS_RESET_net,
                 INIT_POWER_UP => INIT_POWER_UP_net,
                 INIT_DONE => INIT_DONE_net, 
                 INIT_DATA => INIT_DATA_net, 
                 INIT_ADDR => INIT_ADDR_net,
                 INIT_ACM_WEN => INIT_ACM_WEN_net,
                 INIT_ASSC_WEN => INIT_ASSC_WEN_net,
                 INIT_EV_WEN => INIT_EV_WEN_net, 
                 INIT_TR_WEN => INIT_TR_WEN_net);
end RTL;
